#include "execute.hh"

#include <cstring>
#include <fstream>
#include <iostream>

int main(int argc, char **argv)
{
    char *inname = NULL, *tracename = NULL;
    FILE *ftrace = NULL;
    int tracelevel = TRACE_NONE;
    
    uword_t memory[65536];
    
    for (int i = 1; i < argc; i++)
    {
        if (strcmp(argv[i], "-t") == 0)
        {
            if (++i == argc)
            {
                std::cerr << "Option -t requires an argument" << std::endl;
                return 2;
            }
            if ((tracename = strchr(argv[i], ':')))
                *tracename++ = '\0';
            
            if (strcmp(argv[i], "call") == 0)
            {
                tracelevel = TRACE_CALL;
            }
            else if (strcmp(argv[i], "full") == 0)
            {
                tracelevel = TRACE_FULL;
            }
            else
            {
                std::cerr << "Invalid trace level" << std::endl;
                return 2;
            }
        }
        else if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
        {
            printf("%s [-t level[:file]] infile\n", argv[0]);
            printf("%s -h\n", argv[0]);
            printf("\n");
            printf("  Reads b machine code from infile and runs it. With -h, instead output this help.\n");
            printf("  With -t level, write to standard error:\n");
            printf("    every call/return instruction, if level is call or\n");
            printf("    every instruction, if level is full\n");
            printf("  or with -t level:file, write instead to file.\n");
            return 0;
        }
        else
        {
            inname = argv[i];
        }
    }
    
    if (!inname)
    {
        std::cerr << "Input filename is required" << std::endl;
        return 2;
    }
    
    std::ifstream fin(inname);
    if (!fin)
    {
        std::cerr << "Could not open '" << inname << "' for reading" << std::endl;
        return 1;
    }
    fin.read(reinterpret_cast<char *>(memory), sizeof(memory));
    
    if (tracename)
    {
        if (!(ftrace = fopen(tracename, "w")))
        {
            std::cerr << "Could not open '" << tracename << "' for writing" << std::endl;
            return 1;
        }
    }
    else
    {
        ftrace = stderr;
    }
    
    if (!execute(memory, 65536, ftrace, tracelevel))
        return 1;
    
    if (tracename)
        fclose(ftrace);
    
    return 0;
}
