#include "dump.hh"

void put_expression_source(std::ostream &out, expression *expr, int brackets)
{
    switch (expr->type)
    {
        case EXP_FUNCALL:
            put_expression_source(out, expr->operands[0], 1);
            out << "(";
            for (expressions *args = expr->args; args; args = args->next)
            {
                put_expression_source(out, args->expr, 0);
                if (args->next)
                    out << ", ";
            }
            out << ")";
            break;
            
        case EXP_UNARY:
            if (brackets)
                out << "(";
            switch (expr->ival)
            {
                case OP_POSTINC:
                    put_expression_source(out, expr->operands[0], 1);
                    out << "++";
                    break;
                case OP_POSTDEC:
                    put_expression_source(out, expr->operands[0], 1);
                    out << "--";
                    break;
                case OP_PREINC:
                    out << "++";
                    put_expression_source(out, expr->operands[0], 1);
                    break;
                case OP_PREDEC:
                    out << "--";
                    put_expression_source(out, expr->operands[0], 1);
                    break;
                case OP_NEGATE:
                    out << "-";
                    put_expression_source(out, expr->operands[0], 1);
                    break;
                case OP_NOT:
                    out << "!";
                    put_expression_source(out, expr->operands[0], 1);
                    break;
                case OP_COMP:
                    out << "~";
                    put_expression_source(out, expr->operands[0], 1);
                    break;
                case OP_DEREF:
                    if (expr->operands[0]->type == EXP_BINARY && expr->operands[0]->ival == OP_ADD)
                    {
                        put_expression_source(out, expr->operands[0]->operands[0], 1);
                        out << "[";
                        put_expression_source(out, expr->operands[0]->operands[1], 0);
                        out << "]";
                    }
                    else
                    {
                        out << "*";
                        put_expression_source(out, expr->operands[0], 1);
                    }
                    break;
                case OP_ADDR:
                    out << "&";
                    put_expression_source(out, expr->operands[0], 1);
                    break;
            }
            if (brackets)
                out << ")";
            break;
            
        case EXP_BINARY:
            if (brackets)
                out << "(";
            put_expression_source(out, expr->operands[0], 1);
            switch (expr->ival)
            {
                case OP_MODULO: out << " % "; break;
                case OP_MULTIPLY: out << " * "; break;
                case OP_DIVIDE: out << " / "; break;
                case OP_SUBTRACT: out << " - "; break;
                case OP_ADD: out << " + "; break;
                case OP_LEFTSHIFT: out << " << "; break;
                case OP_RIGHTSHIFT: out << " >> "; break;
                case OP_LESS: out << " < "; break;
                case OP_LESSEQUAL: out << " <= "; break;
                case OP_GREATER: out << " > "; break;
                case OP_GREATEREQUAL: out << " >= "; break;
                case OP_EQUAL: out << " == "; break;
                case OP_NOTEQUAL: out << " != "; break;
                case OP_AND: out << " & "; break;
                case OP_XOR: out << " ^ "; break;
                case OP_OR: out << " | "; break;
            }
            put_expression_source(out, expr->operands[1], 1);
            if (brackets)
                out << ")";
            break;
            
        case EXP_ASSIGN:
            if (brackets)
                out << "(";
            put_expression_source(out, expr->operands[0], 1);
            out << " = ";
            put_expression_source(out, expr->operands[1], 1);
            if (brackets)
                out << ")";
            break;
            
        case EXP_COMPOUND:
            if (brackets)
                out << "(";
            put_expression_source(out, expr->operands[0], 1);
            switch (expr->ival)
            {
                case OP_MODULO: out << " =% "; break;
                case OP_MULTIPLY: out << " =* "; break;
                case OP_DIVIDE: out << " =/ "; break;
                case OP_SUBTRACT: out << " =- "; break;
                case OP_ADD: out << " =+ "; break;
                case OP_LEFTSHIFT: out << " =<< "; break;
                case OP_RIGHTSHIFT: out << " =>> "; break;
                case OP_LESS: out << " =< "; break;
                case OP_LESSEQUAL: out << " =<= "; break;
                case OP_GREATER: out << " => "; break;
                case OP_GREATEREQUAL: out << " =>= "; break;
                case OP_EQUAL: out << " === "; break;
                case OP_NOTEQUAL: out << " =!= "; break;
                case OP_AND: out << " =& "; break;
                case OP_XOR: out << " =^ "; break;
                case OP_OR: out << " =| "; break;
            }
            put_expression_source(out, expr->operands[1], 1);
            if (brackets)
                out << ")";
            break;
            
        case EXP_TERNARY:
            if (brackets)
                out << "(";
            put_expression_source(out, expr->operands[0], 1);
            out << " ? ";
            put_expression_source(out, expr->operands[1], 0);
            out << " : ";
            put_expression_source(out, expr->operands[2], 1);
            if (brackets)
                out << ")";
            break;
            
        case EXP_INT:
        case EXP_CHAR:
        case EXP_STRING:
            out << expr->sval;
            break;
            
        case EXP_NAME:
            out << expr->sval;
            break;
    }
}
