#include "ast.hh"

statement::statement(statementtype type) :
    type(type),
    decls(0),
    sval(0),
    expr(0),
    next(0)
{
    contained[0] = 0;
    contained[1] = 0;
}

statement::~statement()
{
    delete decls;
    delete sval;
    delete expr;
    delete contained[0];
    delete contained[1];
    delete next;
}

statement *statement::make_auto(var_decl *decls, statement *contained)
{
    statement *result = new statement(STMT_AUTO);
    result->decls = decls;
    result->contained[0] = contained;
    return result;
}

statement *statement::make_extrn(var_decl *decls, statement *contained)
{
    statement *result = new statement(STMT_EXTRN);
    result->decls = decls;
    result->contained[0] = contained;
    return result;
}

statement *statement::make_labelled(char *label, statement *contained)
{
    statement *result = new statement(STMT_LABELLED);
    result->sval = label;
    result->contained[0] = contained;
    return result;
}

statement *statement::make_case(expression *value, statement *contained)
{
    statement *result = new statement(STMT_CASE);
    result->expr = value;
    result->contained[0] = contained;
    return result;
}

statement *statement::make_block(statement *contained)
{
    statement *result = new statement(STMT_BLOCK);
    result->contained[0] = contained;
    return result;
}

statement *statement::make_if(expression *condition, statement *then, statement *else_)
{
    statement *result = new statement(STMT_IF);
    result->expr = condition;
    result->contained[0] = then;
    result->contained[1] = else_;
    return result;
}

statement *statement::make_while(expression *condition, statement *contained)
{
    statement *result = new statement(STMT_WHILE);
    result->expr = condition;
    result->contained[0] = contained;
    return result;
}

statement *statement::make_break()
{
    statement *result = new statement(STMT_BREAK);
    return result;
}

statement *statement::make_switch(expression *value, statement *contained)
{
    statement *result = new statement(STMT_SWITCH);
    result->expr = value;
    result->contained[0] = contained;
    return result;
}

statement *statement::make_goto(expression *value)
{
    statement *result = new statement(STMT_GOTO);
    result->expr = value;
    return result;
}

statement *statement::make_return(expression *value)
{
    statement *result = new statement(STMT_RETURN);
    result->expr = value;
    return result;
}

statement *statement::make_null()
{
    statement *result = new statement(STMT_NULL);
    return result;
}

statement *statement::make_expression(expression *expr)
{
    statement *result = new statement(STMT_EXPRESSION);
    result->expr = expr;
    return result;
}
